/********************************************************************************

	File:		TrapUtils.C

	Contains:	Functions to help you when working with traps.

	File Ownership:	Apple Macintosh Developer Technical Support
	
	Copyright:	 1996-1998 by Apple Computer, Inc., all rights reserved.

	Bugs?:		For bug reports, consult the following page on
				the World Wide Web:

					http://developer.apple.com/bugreporter/


	========================================================================
	You may incorporate this sample code into your applications without
	restriction, though the sample code has been provided AS IS and the
	responsibility for its operation is 100% yours.  However, what you are
	not permitted to do is to redistribute the source as DSC Sample Code
	after having made changes. If youre going to re-distribute the source,
	we require that you make it clear in the source that the code was
	descended from Apple Sample Code, but that youve made changes.
	========================================================================

	Writers:

		(PG)	Pete Gontier
		(afb)	Andy Bachorski
		
	Change History (most recent first):

		2				afb		Added prefix file for conditional macros
								Changed  ? :  notation to  if else
		1				PG		Initial pass
		
********************************************************************************/


//	Conditionals to setup the build environment the way we like it.
#include "PrivateConditionals.h"


//**********	Universal Headers		****************************************

#include <OSUtils.h>
#include <Traps.h>


//**********	Project Headers			****************************************

#include "TrapUtils.h"


//******************************************************************************

static UInt16 CountToolboxTraps( void )
{
	static UInt16 trapCount;
	
	if ( !trapCount )
	{
		if ( GetToolTrapAddress( _InitGraf ) == GetToolTrapAddress( 0xAA6E ) )
		{
			trapCount = 0x0200;
		}
		else
		{
			trapCount = 0x0400;
		}
	}
	
	return trapCount;
}

//******************************************************************************

static TrapType GetTrapType( UInt16 trapWord )
{
	if ( trapWord & 0x0800 )
	{
		return ToolTrap;
	}
	else
	{
		return OSTrap;
	}
}

//******************************************************************************

pascal Boolean TrapAvailable( UInt16 trapWord )
{
	if ( GetTrapType( trapWord ) == OSTrap )
	{
		return GetToolTrapAddress( _Unimplemented ) != GetOSTrapAddress( trapWord );
	}
	else if ( ( trapWord & 0x07FF ) >= CountToolboxTraps () )
	{
		return false;
	}
	else
	{
		return GetToolTrapAddress( _Unimplemented ) != GetToolTrapAddress ( trapWord );
	}
}

//******************************************************************************

pascal void * MyGetTrapAddress( UInt16 trapWord )
{
	if ( GetTrapType( trapWord ) == OSTrap )
	{
		return GetOSTrapAddress( trapWord );
	}
	else
	{
		return GetToolTrapAddress( trapWord );
	}
}

//******************************************************************************

pascal void MySetTrapAddress( UInt16 trapWord, void *newAddr )
{
	if ( GetTrapType( trapWord ) == OSTrap )
	{
		SetOSTrapAddress( newAddr, trapWord );
	}
	else
	{
		SetToolTrapAddress( newAddr, trapWord );
	}
}

//******************************************************************************
